% function [Q, lamda] = IDHpoint(q, dq, dp, alpha, beta)
%
% Single Point Hamiltonian Inverse Dynamics Function
%
% This solves the inverse dynamics problem for a specific point using the hamiltonian 
% formulation of the equations of motion. Inputs are position, velocity, the first derivative of
% the canonical momenta and the angles alpha and beta. Outputs are the required torque and 
% the lagrange multipliers. While the code is used to calculate one point at a time, it is able to 
% operate on numerous points at once if entered as a q, dq and dp are vectors.
% 
% Input Parameters:
% 		q 		= position vector (position of travelling plate and motor angles)
%  	dq 	= velocity vector (velocity of travelling plate and angular velocity of motors)
% 		dp 	= vector of first derivatives of canonical momentas
%     alpha = the orientation of the motors in the 'alpha' direction, in radians
%     beta  = the orientation of the motors in the 'beta' direction, in radians
%
% Default values:
% 		q, dq, dp, alpha and beta are required
%
% Output values:
%     Q = vector of motor torques
%     lambda = vector of Lagrange multipliers
%
% Required Functions
% 		InitArms
%		InitDynamics
%		
% All distances are in metres, and all angles in radians.
%
% Author: Ben Hawkey 2000


function [Q, lamda] = IDHpoint(q, dq, dp, alpha, beta)


InitArms
InitDynamics

x = q(:,1);
y = q(:,2);
z = q(:,3);
dx = dq(:,1);
dy = dq(:,2);
dz = dq(:,3);
theta1 = q(:,4);
theta2 = q(:,5);
theta3 = q(:,6);
dtheta1 = dq(:,4);
dtheta2 = dq(:,5);
dtheta3 = dq(:,6);
dpx = dp(:,1);
dpy = dp(:,2);
dpz = dp(:,3);
dptheta1 = dp(:,4);
dptheta2 = dp(:,5);
dptheta3 = dp(:,6);

R = Ra-Rb;

cosalpha = cos(alpha);
sinalpha = sin(alpha);
cosbeta = cos(beta);
sinbeta = sin(beta);

costheta1 = cos(theta1);
sintheta1 = sin(theta1);
costheta2 = cos(theta2);
sintheta2 = sin(theta2);
costheta3 = cos(theta3);
sintheta3 = sin(theta3);

% All of the trigonometric expressions in the equations are replaced with constants
% so as to reduce the number of times they must be evaluated.



lamda1 = (-1).*(dpz+(1/2).*g.*(3.*Mb+2.*Mc)).*(2.*cosalpha.*La.*sintheta1+ ...
2.*z).^(-1)+(2.*cosalpha.*La.*sintheta1+2.*z).^(-1).*(2.* ...
cosalpha.*La.*sintheta3+2.*z).*((-1).*((-1).*(dpz+(1/2).*g.*(3.* ...
Mb+2.*Mc)).*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.*La.* ...
sinalpha.*sintheta1+2.*y)+dpy.*(2.*cosalpha.*La.*sintheta1+2.*z)) ...
.*((R+(-1).*La.*sinbeta.*(sqrt(3).*costheta2+(-1).*sinalpha.* ...
sintheta2)+(-1).*cosbeta.*La.*((-1).*costheta2+(-1).*sqrt(3).* ...
sinalpha.*sintheta2)+2.*x).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1) ...
.*((-2).*R+(-2).*cosbeta.*costheta1.*La+(-2).*La.*sinalpha.* ...
sinbeta.*sintheta1+2.*x).*(2.*cosalpha.*La.*sintheta2+2.*z))+((-1) ...
.*(dpz+(1/2).*g.*(3.*Mb+2.*Mc)).*((-2).*R+(-2).*cosbeta.* ...
costheta1.*La+(-2).*La.*sinalpha.*sinbeta.*sintheta1+2.*x)+dpx.*( ...
2.*cosalpha.*La.*sintheta1+2.*z)).*(((-1).*sqrt(3).*R+(-1).* ...
cosbeta.*La.*(sqrt(3).*costheta2+(-1).*sinalpha.*sintheta2)+(-1).* ...
La.*sinbeta.*(costheta2+sqrt(3).*sinalpha.*sintheta2)+2.*y).*(2.* ...
cosalpha.*La.*sintheta1+2.*z)+(-1).*(2.*costheta1.*La.*sinbeta+( ...
-2).*cosbeta.*La.*sinalpha.*sintheta1+2.*y).*(2.*cosalpha.*La.* ...
sintheta2+2.*z))).*((((-1).*sqrt(3).*R+(-1).*cosbeta.*La.*(sqrt(3) ...
.*costheta2+(-1).*sinalpha.*sintheta2)+(-1).*La.*sinbeta.*( ...
costheta2+sqrt(3).*sinalpha.*sintheta2)+2.*y).*(2.*cosalpha.*La.* ...
sintheta1+2.*z)+(-1).*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.* ...
La.*sinalpha.*sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta2+2.*z)) ...
.*((R+(-1).*La.*sinbeta.*((-1).*sqrt(3).*costheta3+(-1).* ...
sinalpha.*sintheta3)+(-1).*cosbeta.*La.*((-1).*costheta3+sqrt(3).* ...
sinalpha.*sintheta3)+2.*x).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1) ...
.*((-2).*R+(-2).*cosbeta.*costheta1.*La+(-2).*La.*sinalpha.* ...
sinbeta.*sintheta1+2.*x).*(2.*cosalpha.*La.*sintheta3+2.*z))+(-1) ...
.*((R+(-1).*La.*sinbeta.*(sqrt(3).*costheta2+(-1).*sinalpha.* ...
sintheta2)+(-1).*cosbeta.*La.*((-1).*costheta2+(-1).*sqrt(3).* ...
sinalpha.*sintheta2)+2.*x).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1) ...
.*((-2).*R+(-2).*cosbeta.*costheta1.*La+(-2).*La.*sinalpha.* ...
sinbeta.*sintheta1+2.*x).*(2.*cosalpha.*La.*sintheta2+2.*z)).*(( ...
sqrt(3).*R+(-1).*cosbeta.*La.*((-1).*sqrt(3).*costheta3+(-1).* ...
sinalpha.*sintheta3)+(-1).*La.*sinbeta.*(costheta3+(-1).*sqrt(3).* ...
sinalpha.*sintheta3)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1) ...
.*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.* ...
sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta3+2.*z))).^(-1)+(2.* ...
cosalpha.*La.*sintheta1+2.*z).^(-1).*(2.*cosalpha.*La.*sintheta2+ ...
2.*z).*(((-1).*(dpz+(1/2).*g.*(3.*Mb+2.*Mc)).*(2.*costheta1.*La.* ...
sinbeta+(-2).*cosbeta.*La.*sinalpha.*sintheta1+2.*y)+dpy.*(2.* ...
cosalpha.*La.*sintheta1+2.*z)).*(((-1).*sqrt(3).*R+(-1).*cosbeta.* ...
La.*(sqrt(3).*costheta2+(-1).*sinalpha.*sintheta2)+(-1).*La.* ...
sinbeta.*(costheta2+sqrt(3).*sinalpha.*sintheta2)+2.*y).*(2.* ...
cosalpha.*La.*sintheta1+2.*z)+(-1).*(2.*costheta1.*La.*sinbeta+( ...
-2).*cosbeta.*La.*sinalpha.*sintheta1+2.*y).*(2.*cosalpha.*La.* ...
sintheta2+2.*z)).^(-1)+(-1).*(((-1).*sqrt(3).*R+(-1).*cosbeta.* ...
La.*(sqrt(3).*costheta2+(-1).*sinalpha.*sintheta2)+(-1).*La.* ...
sinbeta.*(costheta2+sqrt(3).*sinalpha.*sintheta2)+2.*y).*(2.* ...
cosalpha.*La.*sintheta1+2.*z)+(-1).*(2.*costheta1.*La.*sinbeta+( ...
-2).*cosbeta.*La.*sinalpha.*sintheta1+2.*y).*(2.*cosalpha.*La.* ...
sintheta2+2.*z)).^(-1).*((sqrt(3).*R+(-1).*cosbeta.*La.*((-1).* ...
sqrt(3).*costheta3+(-1).*sinalpha.*sintheta3)+(-1).*La.*sinbeta.*( ...
costheta3+(-1).*sqrt(3).*sinalpha.*sintheta3)+2.*y).*(2.* ...
cosalpha.*La.*sintheta1+2.*z)+(-1).*(2.*costheta1.*La.*sinbeta+( ...
-2).*cosbeta.*La.*sinalpha.*sintheta1+2.*y).*(2.*cosalpha.*La.* ...
sintheta3+2.*z)).*((-1).*((-1).*(dpz+(1/2).*g.*(3.*Mb+2.*Mc)).*( ...
2.*costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.*sintheta1+ ...
2.*y)+dpy.*(2.*cosalpha.*La.*sintheta1+2.*z)).*((R+(-1).*La.* ...
sinbeta.*(sqrt(3).*costheta2+(-1).*sinalpha.*sintheta2)+(-1).* ...
cosbeta.*La.*((-1).*costheta2+(-1).*sqrt(3).*sinalpha.*sintheta2)+ ...
2.*x).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1).*((-2).*R+(-2).* ...
cosbeta.*costheta1.*La+(-2).*La.*sinalpha.*sinbeta.*sintheta1+2.* ...
x).*(2.*cosalpha.*La.*sintheta2+2.*z))+((-1).*(dpz+(1/2).*g.*(3.* ...
Mb+2.*Mc)).*((-2).*R+(-2).*cosbeta.*costheta1.*La+(-2).*La.* ...
sinalpha.*sinbeta.*sintheta1+2.*x)+dpx.*(2.*cosalpha.*La.* ...
sintheta1+2.*z)).*(((-1).*sqrt(3).*R+(-1).*cosbeta.*La.*(sqrt(3).* ...
costheta2+(-1).*sinalpha.*sintheta2)+(-1).*La.*sinbeta.*( ...
costheta2+sqrt(3).*sinalpha.*sintheta2)+2.*y).*(2.*cosalpha.*La.* ...
sintheta1+2.*z)+(-1).*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.* ...
La.*sinalpha.*sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta2+2.*z))) ...
.*((((-1).*sqrt(3).*R+(-1).*cosbeta.*La.*(sqrt(3).*costheta2+(-1) ...
.*sinalpha.*sintheta2)+(-1).*La.*sinbeta.*(costheta2+sqrt(3).* ...
sinalpha.*sintheta2)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1) ...
.*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.* ...
sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta2+2.*z)).*((R+(-1).* ...
La.*sinbeta.*((-1).*sqrt(3).*costheta3+(-1).*sinalpha.*sintheta3)+ ...
(-1).*cosbeta.*La.*((-1).*costheta3+sqrt(3).*sinalpha.*sintheta3)+ ...
2.*x).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1).*((-2).*R+(-2).* ...
cosbeta.*costheta1.*La+(-2).*La.*sinalpha.*sinbeta.*sintheta1+2.* ...
x).*(2.*cosalpha.*La.*sintheta3+2.*z))+(-1).*((R+(-1).*La.* ...
sinbeta.*(sqrt(3).*costheta2+(-1).*sinalpha.*sintheta2)+(-1).* ...
cosbeta.*La.*((-1).*costheta2+(-1).*sqrt(3).*sinalpha.*sintheta2)+ ...
2.*x).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1).*((-2).*R+(-2).* ...
cosbeta.*costheta1.*La+(-2).*La.*sinalpha.*sinbeta.*sintheta1+2.* ...
x).*(2.*cosalpha.*La.*sintheta2+2.*z)).*((sqrt(3).*R+(-1).* ...
cosbeta.*La.*((-1).*sqrt(3).*costheta3+(-1).*sinalpha.*sintheta3)+ ...
(-1).*La.*sinbeta.*(costheta3+(-1).*sqrt(3).*sinalpha.*sintheta3)+ ...
2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1).*(2.*costheta1.*La.* ...
sinbeta+(-2).*cosbeta.*La.*sinalpha.*sintheta1+2.*y).*(2.* ...
cosalpha.*La.*sintheta3+2.*z))).^(-1));

lamda2 = (-1).*((-1).*(dpz+(1/2).*g.*(3.*Mb+2.*Mc)).*(2.*costheta1.*La.* ...
sinbeta+(-2).*cosbeta.*La.*sinalpha.*sintheta1+2.*y)+dpy.*(2.* ...
cosalpha.*La.*sintheta1+2.*z)).*(((-1).*sqrt(3).*R+(-1).*cosbeta.* ...
La.*(sqrt(3).*costheta2+(-1).*sinalpha.*sintheta2)+(-1).*La.* ...
sinbeta.*(costheta2+sqrt(3).*sinalpha.*sintheta2)+2.*y).*(2.* ...
cosalpha.*La.*sintheta1+2.*z)+(-1).*(2.*costheta1.*La.*sinbeta+( ...
-2).*cosbeta.*La.*sinalpha.*sintheta1+2.*y).*(2.*cosalpha.*La.* ...
sintheta2+2.*z)).^(-1)+(((-1).*sqrt(3).*R+(-1).*cosbeta.*La.*( ...
sqrt(3).*costheta2+(-1).*sinalpha.*sintheta2)+(-1).*La.*sinbeta.*( ...
costheta2+sqrt(3).*sinalpha.*sintheta2)+2.*y).*(2.*cosalpha.*La.* ...
sintheta1+2.*z)+(-1).*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.* ...
La.*sinalpha.*sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta2+2.*z)) ...
.^(-1).*((sqrt(3).*R+(-1).*cosbeta.*La.*((-1).*sqrt(3).*costheta3+ ...
(-1).*sinalpha.*sintheta3)+(-1).*La.*sinbeta.*(costheta3+(-1).* ...
sqrt(3).*sinalpha.*sintheta3)+2.*y).*(2.*cosalpha.*La.*sintheta1+ ...
2.*z)+(-1).*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.*La.* ...
sinalpha.*sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta3+2.*z)).*(( ...
-1).*((-1).*(dpz+(1/2).*g.*(3.*Mb+2.*Mc)).*(2.*costheta1.*La.* ...
sinbeta+(-2).*cosbeta.*La.*sinalpha.*sintheta1+2.*y)+dpy.*(2.* ...
cosalpha.*La.*sintheta1+2.*z)).*((R+(-1).*La.*sinbeta.*(sqrt(3).* ...
costheta2+(-1).*sinalpha.*sintheta2)+(-1).*cosbeta.*La.*((-1).* ...
costheta2+(-1).*sqrt(3).*sinalpha.*sintheta2)+2.*x).*(2.* ...
cosalpha.*La.*sintheta1+2.*z)+(-1).*((-2).*R+(-2).*cosbeta.* ...
costheta1.*La+(-2).*La.*sinalpha.*sinbeta.*sintheta1+2.*x).*(2.* ...
cosalpha.*La.*sintheta2+2.*z))+((-1).*(dpz+(1/2).*g.*(3.*Mb+2.*Mc) ...
).*((-2).*R+(-2).*cosbeta.*costheta1.*La+(-2).*La.*sinalpha.* ...
sinbeta.*sintheta1+2.*x)+dpx.*(2.*cosalpha.*La.*sintheta1+2.*z)).* ...
(((-1).*sqrt(3).*R+(-1).*cosbeta.*La.*(sqrt(3).*costheta2+(-1).* ...
sinalpha.*sintheta2)+(-1).*La.*sinbeta.*(costheta2+sqrt(3).* ...
sinalpha.*sintheta2)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1) ...
.*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.* ...
sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta2+2.*z))).*((((-1).* ...
sqrt(3).*R+(-1).*cosbeta.*La.*(sqrt(3).*costheta2+(-1).*sinalpha.* ...
sintheta2)+(-1).*La.*sinbeta.*(costheta2+sqrt(3).*sinalpha.* ...
sintheta2)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1).*(2.* ...
costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.*sintheta1+2.* ...
y).*(2.*cosalpha.*La.*sintheta2+2.*z)).*((R+(-1).*La.*sinbeta.*(( ...
-1).*sqrt(3).*costheta3+(-1).*sinalpha.*sintheta3)+(-1).*cosbeta.* ...
La.*((-1).*costheta3+sqrt(3).*sinalpha.*sintheta3)+2.*x).*(2.* ...
cosalpha.*La.*sintheta1+2.*z)+(-1).*((-2).*R+(-2).*cosbeta.* ...
costheta1.*La+(-2).*La.*sinalpha.*sinbeta.*sintheta1+2.*x).*(2.* ...
cosalpha.*La.*sintheta3+2.*z))+(-1).*((R+(-1).*La.*sinbeta.*(sqrt( ...
3).*costheta2+(-1).*sinalpha.*sintheta2)+(-1).*cosbeta.*La.*((-1) ...
.*costheta2+(-1).*sqrt(3).*sinalpha.*sintheta2)+2.*x).*(2.* ...
cosalpha.*La.*sintheta1+2.*z)+(-1).*((-2).*R+(-2).*cosbeta.* ...
costheta1.*La+(-2).*La.*sinalpha.*sinbeta.*sintheta1+2.*x).*(2.* ...
cosalpha.*La.*sintheta2+2.*z)).*((sqrt(3).*R+(-1).*cosbeta.*La.*(( ...
-1).*sqrt(3).*costheta3+(-1).*sinalpha.*sintheta3)+(-1).*La.* ...
sinbeta.*(costheta3+(-1).*sqrt(3).*sinalpha.*sintheta3)+2.*y).*( ...
2.*cosalpha.*La.*sintheta1+2.*z)+(-1).*(2.*costheta1.*La.*sinbeta+ ...
(-2).*cosbeta.*La.*sinalpha.*sintheta1+2.*y).*(2.*cosalpha.*La.* ...
sintheta3+2.*z))).^(-1);

lamda3 = (-1).*((-1).*((-1).*(dpz+(1/2).*g.*(3.*Mb+2.*Mc)).*(2.*costheta1.* ...
La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.*sintheta1+2.*y)+dpy.*(2.* ...
cosalpha.*La.*sintheta1+2.*z)).*((R+(-1).*La.*sinbeta.*(sqrt(3).* ...
costheta2+(-1).*sinalpha.*sintheta2)+(-1).*cosbeta.*La.*((-1).* ...
costheta2+(-1).*sqrt(3).*sinalpha.*sintheta2)+2.*x).*(2.* ...
cosalpha.*La.*sintheta1+2.*z)+(-1).*((-2).*R+(-2).*cosbeta.* ...
costheta1.*La+(-2).*La.*sinalpha.*sinbeta.*sintheta1+2.*x).*(2.* ...
cosalpha.*La.*sintheta2+2.*z))+((-1).*(dpz+(1/2).*g.*(3.*Mb+2.*Mc) ...
).*((-2).*R+(-2).*cosbeta.*costheta1.*La+(-2).*La.*sinalpha.* ...
sinbeta.*sintheta1+2.*x)+dpx.*(2.*cosalpha.*La.*sintheta1+2.*z)).* ...
(((-1).*sqrt(3).*R+(-1).*cosbeta.*La.*(sqrt(3).*costheta2+(-1).* ...
sinalpha.*sintheta2)+(-1).*La.*sinbeta.*(costheta2+sqrt(3).* ...
sinalpha.*sintheta2)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1) ...
.*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.* ...
sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta2+2.*z))).*((((-1).* ...
sqrt(3).*R+(-1).*cosbeta.*La.*(sqrt(3).*costheta2+(-1).*sinalpha.* ...
sintheta2)+(-1).*La.*sinbeta.*(costheta2+sqrt(3).*sinalpha.* ...
sintheta2)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1).*(2.* ...
costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.*sintheta1+2.* ...
y).*(2.*cosalpha.*La.*sintheta2+2.*z)).*((R+(-1).*La.*sinbeta.*(( ...
-1).*sqrt(3).*costheta3+(-1).*sinalpha.*sintheta3)+(-1).*cosbeta.* ...
La.*((-1).*costheta3+sqrt(3).*sinalpha.*sintheta3)+2.*x).*(2.* ...
cosalpha.*La.*sintheta1+2.*z)+(-1).*((-2).*R+(-2).*cosbeta.* ...
costheta1.*La+(-2).*La.*sinalpha.*sinbeta.*sintheta1+2.*x).*(2.* ...
cosalpha.*La.*sintheta3+2.*z))+(-1).*((R+(-1).*La.*sinbeta.*(sqrt( ...
3).*costheta2+(-1).*sinalpha.*sintheta2)+(-1).*cosbeta.*La.*((-1) ...
.*costheta2+(-1).*sqrt(3).*sinalpha.*sintheta2)+2.*x).*(2.* ...
cosalpha.*La.*sintheta1+2.*z)+(-1).*((-2).*R+(-2).*cosbeta.* ...
costheta1.*La+(-2).*La.*sinalpha.*sinbeta.*sintheta1+2.*x).*(2.* ...
cosalpha.*La.*sintheta2+2.*z)).*((sqrt(3).*R+(-1).*cosbeta.*La.*(( ...
-1).*sqrt(3).*costheta3+(-1).*sinalpha.*sintheta3)+(-1).*La.* ...
sinbeta.*(costheta3+(-1).*sqrt(3).*sinalpha.*sintheta3)+2.*y).*( ...
2.*cosalpha.*La.*sintheta1+2.*z)+(-1).*(2.*costheta1.*La.*sinbeta+ ...
(-2).*cosbeta.*La.*sinalpha.*sintheta1+2.*y).*(2.*cosalpha.*La.* ...
sintheta3+2.*z))).^(-1);

lamda = [lamda1 lamda2 lamda3];

Q4 = dptheta1+(1/6).*((-3).*cosalpha.*costheta1.*g.*La.*(2.*H.*Ma+Mb+ ...
2.*Mj)+dtheta1.*La.*Mb.*((-1).*cosalpha.*dz.*sintheta1+dy.*((-1).* ...
costheta1.*sinbeta+cosbeta.*sinalpha.*sintheta1)+dx.*(cosbeta.* ...
costheta1+sinalpha.*sinbeta.*sintheta1)))+2.*La.*lamda1.*( ...
costheta1.*R.*sinalpha.*sinbeta+(-1).*cosbeta.*R.*sintheta1+((-1) ...
.*costheta1.*sinalpha.*sinbeta+cosbeta.*sintheta1).*x+(-1).*( ...
cosbeta.*costheta1.*sinalpha+sinbeta.*sintheta1).*y+cosalpha.* ...
costheta1.*z);

Q5 = dptheta2+(-1/12).*La.*(6.*cosalpha.*costheta2.*g.*(2.*H.*Ma+Mb+2.* ...
Mj)+dtheta2.*Mb.*(costheta2.*((-1).*dy.*(sqrt(3).*cosbeta+sinbeta) ...
+dx.*(cosbeta+(-1).*sqrt(3).*sinbeta))+(2.*cosalpha.*dz+dx.* ...
sinalpha.*(sqrt(3).*cosbeta+sinbeta)+dy.*sinalpha.*(cosbeta+(-1).* ...
sqrt(3).*sinbeta)).*sintheta2))+La.*lamda2.*(sintheta2.*(sinbeta.* ...
(sqrt(3).*x+y)+(-1).*cosbeta.*(2.*R+x+(-1).*sqrt(3).*y))+ ...
costheta2.*sinalpha.*(cosbeta.*(sqrt(3).*x+y)+sinbeta.*(2.*R+x+( ...
-1).*sqrt(3).*y))+2.*cosalpha.*costheta2.*z);

Q6 = dptheta3+(-1/12).*La.*(6.*cosalpha.*costheta3.*g.*(2.*H.*Ma+Mb+2.* ...
Mj)+dtheta3.*Mb.*(costheta3.*(dy.*(sqrt(3).*cosbeta+(-1).*sinbeta) ...
+dx.*(cosbeta+sqrt(3).*sinbeta))+(2.*cosalpha.*dz+sinalpha.*(dx.*( ...
(-1).*sqrt(3).*cosbeta+sinbeta)+dy.*(cosbeta+sqrt(3).*sinbeta))).* ...
sintheta3))+La.*lamda3.*((-1).*sintheta3.*(sinbeta.*(sqrt(3).*x+( ...
-1).*y)+cosbeta.*(2.*R+x+sqrt(3).*y))+costheta3.*sinalpha.*( ...
cosbeta.*((-1).*sqrt(3).*x+y)+sinbeta.*(2.*R+x+sqrt(3).*y))+2.* ...
cosalpha.*costheta3.*z);

Q = [Q4, Q5, Q6];

