%[lamda]=lagrange_multipliers(q, dp, alpha, beta)

% Lagrange multiplier function
%
% This function is used by the IDHtraj function to calculate lagrange multipiers for the whole trajectory in one operation. 
% It can be used on a point by point basis however. 
%
% Input parameters:
% q = matrix of travelling plate position and motor angles.
% dq = matrix of travelling plate velocity and motor angular velocity.
%
% Outputs:
% lamda = lagrange multipliers for entire time history.
%
% Author: Ben Hawkey 2000.


function[lamda]=lagrange_multipliers(q, dp, alpha, beta)

InitArms
InitDynamics

x = q(:,1);
y = q(:,2);
z = q(:,3);
theta1 = q(:,4);
theta2 = q(:,5);
theta3 = q(:,6);
dpx = dp(:,1);
dpy = dp(:,2);
dpz = dp(:,3);

R = Ra-Rb;

cosalpha = cos(alpha);
sinalpha = sin(alpha);
cosbeta = cos(beta);
sinbeta = sin(beta);

costheta1 = cos(theta1);
sintheta1 = sin(theta1);
costheta2 = cos(theta2);
sintheta2 = sin(theta2);
costheta3 = cos(theta3);
sintheta3 = sin(theta3);

% All of the trigonometric expressions in the equations are replaced with constants
% so as to reduce the number of times they must be evaluated.

lamda1 = (-1).*(dpz+(1/2).*g.*(3.*Mb+2.*Mc)).*(2.*cosalpha.*La.*sintheta1+ ...
2.*z).^(-1)+(2.*cosalpha.*La.*sintheta1+2.*z).^(-1).*(2.* ...
cosalpha.*La.*sintheta3+2.*z).*((-1).*((-1).*(dpz+(1/2).*g.*(3.* ...
Mb+2.*Mc)).*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.*La.* ...
sinalpha.*sintheta1+2.*y)+dpy.*(2.*cosalpha.*La.*sintheta1+2.*z)) ...
.*((R+(-1).*sqrt(3).*costheta2.*La.*sinbeta+La.*sinalpha.* ...
sinbeta.*sintheta2+cosbeta.*La.*(costheta2+sqrt(3).*sinalpha.* ...
sintheta2)+2.*x).*(2.*cosalpha.*La.*sintheta1+2.*z)+2.*(R+ ...
cosbeta.*costheta1.*La+La.*sinalpha.*sinbeta.*sintheta1+(-1).*x).* ...
(2.*cosalpha.*La.*sintheta2+2.*z))+(2.*(dpz+(1/2).*g.*(3.*Mb+2.* ...
Mc)).*(R+cosbeta.*costheta1.*La+La.*sinalpha.*sinbeta.*sintheta1+( ...
-1).*x)+dpx.*(2.*cosalpha.*La.*sintheta1+2.*z)).*(((-1).*sqrt(3).* ...
R+(-1).*costheta2.*La.*sinbeta+(-1).*sqrt(3).*La.*sinalpha.* ...
sinbeta.*sintheta2+(-1).*cosbeta.*La.*(sqrt(3).*costheta2+(-1).* ...
sinalpha.*sintheta2)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1) ...
.*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.* ...
sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta2+2.*z))).*((((-1).* ...
sqrt(3).*R+(-1).*costheta2.*La.*sinbeta+(-1).*sqrt(3).*La.* ...
sinalpha.*sinbeta.*sintheta2+(-1).*cosbeta.*La.*(sqrt(3).* ...
costheta2+(-1).*sinalpha.*sintheta2)+2.*y).*(2.*cosalpha.*La.* ...
sintheta1+2.*z)+(-1).*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.* ...
La.*sinalpha.*sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta2+2.*z)) ...
.*((R+sqrt(3).*costheta3.*La.*sinbeta+La.*sinalpha.*sinbeta.* ...
sintheta3+cosbeta.*La.*(costheta3+(-1).*sqrt(3).*sinalpha.* ...
sintheta3)+2.*x).*(2.*cosalpha.*La.*sintheta1+2.*z)+2.*(R+ ...
cosbeta.*costheta1.*La+La.*sinalpha.*sinbeta.*sintheta1+(-1).*x).* ...
(2.*cosalpha.*La.*sintheta3+2.*z))+(-1).*((R+(-1).*sqrt(3).* ...
costheta2.*La.*sinbeta+La.*sinalpha.*sinbeta.*sintheta2+cosbeta.* ...
La.*(costheta2+sqrt(3).*sinalpha.*sintheta2)+2.*x).*(2.*cosalpha.* ...
La.*sintheta1+2.*z)+2.*(R+cosbeta.*costheta1.*La+La.*sinalpha.* ...
sinbeta.*sintheta1+(-1).*x).*(2.*cosalpha.*La.*sintheta2+2.*z)).*( ...
(sqrt(3).*R+(-1).*costheta3.*La.*sinbeta+sqrt(3).*La.*sinalpha.* ...
sinbeta.*sintheta3+(-1).*cosbeta.*La.*((-1).*sqrt(3).*costheta3+( ...
-1).*sinalpha.*sintheta3)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z) ...
+(-1).*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.* ...
sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta3+2.*z))).^(-1)+(2.* ...
cosalpha.*La.*sintheta1+2.*z).^(-1).*(2.*cosalpha.*La.*sintheta2+ ...
2.*z).*(((-1).*(dpz+(1/2).*g.*(3.*Mb+2.*Mc)).*(2.*costheta1.*La.* ...
sinbeta+(-2).*cosbeta.*La.*sinalpha.*sintheta1+2.*y)+dpy.*(2.* ...
cosalpha.*La.*sintheta1+2.*z)).*(((-1).*sqrt(3).*R+(-1).* ...
costheta2.*La.*sinbeta+(-1).*sqrt(3).*La.*sinalpha.*sinbeta.* ...
sintheta2+(-1).*cosbeta.*La.*(sqrt(3).*costheta2+(-1).*sinalpha.* ...
sintheta2)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1).*(2.* ...
costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.*sintheta1+2.* ...
y).*(2.*cosalpha.*La.*sintheta2+2.*z)).^(-1)+(-1).*(((-1).*sqrt(3) ...
.*R+(-1).*costheta2.*La.*sinbeta+(-1).*sqrt(3).*La.*sinalpha.* ...
sinbeta.*sintheta2+(-1).*cosbeta.*La.*(sqrt(3).*costheta2+(-1).* ...
sinalpha.*sintheta2)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1) ...
.*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.* ...
sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta2+2.*z)).^(-1).*((sqrt( ...
3).*R+(-1).*costheta3.*La.*sinbeta+sqrt(3).*La.*sinalpha.* ...
sinbeta.*sintheta3+(-1).*cosbeta.*La.*((-1).*sqrt(3).*costheta3+( ...
-1).*sinalpha.*sintheta3)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z) ...
+(-1).*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.* ...
sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta3+2.*z)).*((-1).*((-1) ...
.*(dpz+(1/2).*g.*(3.*Mb+2.*Mc)).*(2.*costheta1.*La.*sinbeta+(-2).* ...
cosbeta.*La.*sinalpha.*sintheta1+2.*y)+dpy.*(2.*cosalpha.*La.* ...
sintheta1+2.*z)).*((R+(-1).*sqrt(3).*costheta2.*La.*sinbeta+La.* ...
sinalpha.*sinbeta.*sintheta2+cosbeta.*La.*(costheta2+sqrt(3).* ...
sinalpha.*sintheta2)+2.*x).*(2.*cosalpha.*La.*sintheta1+2.*z)+2.*( ...
R+cosbeta.*costheta1.*La+La.*sinalpha.*sinbeta.*sintheta1+(-1).*x) ...
.*(2.*cosalpha.*La.*sintheta2+2.*z))+(2.*(dpz+(1/2).*g.*(3.*Mb+2.* ...
Mc)).*(R+cosbeta.*costheta1.*La+La.*sinalpha.*sinbeta.*sintheta1+( ...
-1).*x)+dpx.*(2.*cosalpha.*La.*sintheta1+2.*z)).*(((-1).*sqrt(3).* ...
R+(-1).*costheta2.*La.*sinbeta+(-1).*sqrt(3).*La.*sinalpha.* ...
sinbeta.*sintheta2+(-1).*cosbeta.*La.*(sqrt(3).*costheta2+(-1).* ...
sinalpha.*sintheta2)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1) ...
.*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.* ...
sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta2+2.*z))).*((((-1).* ...
sqrt(3).*R+(-1).*costheta2.*La.*sinbeta+(-1).*sqrt(3).*La.* ...
sinalpha.*sinbeta.*sintheta2+(-1).*cosbeta.*La.*(sqrt(3).* ...
costheta2+(-1).*sinalpha.*sintheta2)+2.*y).*(2.*cosalpha.*La.* ...
sintheta1+2.*z)+(-1).*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.* ...
La.*sinalpha.*sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta2+2.*z)) ...
.*((R+sqrt(3).*costheta3.*La.*sinbeta+La.*sinalpha.*sinbeta.* ...
sintheta3+cosbeta.*La.*(costheta3+(-1).*sqrt(3).*sinalpha.* ...
sintheta3)+2.*x).*(2.*cosalpha.*La.*sintheta1+2.*z)+2.*(R+ ...
cosbeta.*costheta1.*La+La.*sinalpha.*sinbeta.*sintheta1+(-1).*x).* ...
(2.*cosalpha.*La.*sintheta3+2.*z))+(-1).*((R+(-1).*sqrt(3).* ...
costheta2.*La.*sinbeta+La.*sinalpha.*sinbeta.*sintheta2+cosbeta.* ...
La.*(costheta2+sqrt(3).*sinalpha.*sintheta2)+2.*x).*(2.*cosalpha.* ...
La.*sintheta1+2.*z)+2.*(R+cosbeta.*costheta1.*La+La.*sinalpha.* ...
sinbeta.*sintheta1+(-1).*x).*(2.*cosalpha.*La.*sintheta2+2.*z)).*( ...
(sqrt(3).*R+(-1).*costheta3.*La.*sinbeta+sqrt(3).*La.*sinalpha.* ...
sinbeta.*sintheta3+(-1).*cosbeta.*La.*((-1).*sqrt(3).*costheta3+( ...
-1).*sinalpha.*sintheta3)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z) ...
+(-1).*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.* ...
sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta3+2.*z))).^(-1));


lamda2 = (-1).*((-1).*(dpz+(1/2).*g.*(3.*Mb+2.*Mc)).*(2.*costheta1.*La.* ...
sinbeta+(-2).*cosbeta.*La.*sinalpha.*sintheta1+2.*y)+dpy.*(2.* ...
cosalpha.*La.*sintheta1+2.*z)).*(((-1).*sqrt(3).*R+(-1).* ...
costheta2.*La.*sinbeta+(-1).*sqrt(3).*La.*sinalpha.*sinbeta.* ...
sintheta2+(-1).*cosbeta.*La.*(sqrt(3).*costheta2+(-1).*sinalpha.* ...
sintheta2)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1).*(2.* ...
costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.*sintheta1+2.* ...
y).*(2.*cosalpha.*La.*sintheta2+2.*z)).^(-1)+(((-1).*sqrt(3).*R+( ...
-1).*costheta2.*La.*sinbeta+(-1).*sqrt(3).*La.*sinalpha.*sinbeta.* ...
sintheta2+(-1).*cosbeta.*La.*(sqrt(3).*costheta2+(-1).*sinalpha.* ...
sintheta2)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1).*(2.* ...
costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.*sintheta1+2.* ...
y).*(2.*cosalpha.*La.*sintheta2+2.*z)).^(-1).*((sqrt(3).*R+(-1).* ...
costheta3.*La.*sinbeta+sqrt(3).*La.*sinalpha.*sinbeta.*sintheta3+( ...
-1).*cosbeta.*La.*((-1).*sqrt(3).*costheta3+(-1).*sinalpha.* ...
sintheta3)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1).*(2.* ...
costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.*sintheta1+2.* ...
y).*(2.*cosalpha.*La.*sintheta3+2.*z)).*((-1).*((-1).*(dpz+(1/2).* ...
g.*(3.*Mb+2.*Mc)).*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.*La.* ...
sinalpha.*sintheta1+2.*y)+dpy.*(2.*cosalpha.*La.*sintheta1+2.*z)) ...
.*((R+(-1).*sqrt(3).*costheta2.*La.*sinbeta+La.*sinalpha.* ...
sinbeta.*sintheta2+cosbeta.*La.*(costheta2+sqrt(3).*sinalpha.* ...
sintheta2)+2.*x).*(2.*cosalpha.*La.*sintheta1+2.*z)+2.*(R+ ...
cosbeta.*costheta1.*La+La.*sinalpha.*sinbeta.*sintheta1+(-1).*x).* ...
(2.*cosalpha.*La.*sintheta2+2.*z))+(2.*(dpz+(1/2).*g.*(3.*Mb+2.* ...
Mc)).*(R+cosbeta.*costheta1.*La+La.*sinalpha.*sinbeta.*sintheta1+( ...
-1).*x)+dpx.*(2.*cosalpha.*La.*sintheta1+2.*z)).*(((-1).*sqrt(3).* ...
R+(-1).*costheta2.*La.*sinbeta+(-1).*sqrt(3).*La.*sinalpha.* ...
sinbeta.*sintheta2+(-1).*cosbeta.*La.*(sqrt(3).*costheta2+(-1).* ...
sinalpha.*sintheta2)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z)+(-1) ...
.*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.* ...
sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta2+2.*z))).*((((-1).* ...
sqrt(3).*R+(-1).*costheta2.*La.*sinbeta+(-1).*sqrt(3).*La.* ...
sinalpha.*sinbeta.*sintheta2+(-1).*cosbeta.*La.*(sqrt(3).* ...
costheta2+(-1).*sinalpha.*sintheta2)+2.*y).*(2.*cosalpha.*La.* ...
sintheta1+2.*z)+(-1).*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.* ...
La.*sinalpha.*sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta2+2.*z)) ...
.*((R+sqrt(3).*costheta3.*La.*sinbeta+La.*sinalpha.*sinbeta.* ...
sintheta3+cosbeta.*La.*(costheta3+(-1).*sqrt(3).*sinalpha.* ...
sintheta3)+2.*x).*(2.*cosalpha.*La.*sintheta1+2.*z)+2.*(R+ ...
cosbeta.*costheta1.*La+La.*sinalpha.*sinbeta.*sintheta1+(-1).*x).* ...
(2.*cosalpha.*La.*sintheta3+2.*z))+(-1).*((R+(-1).*sqrt(3).* ...
costheta2.*La.*sinbeta+La.*sinalpha.*sinbeta.*sintheta2+cosbeta.* ...
La.*(costheta2+sqrt(3).*sinalpha.*sintheta2)+2.*x).*(2.*cosalpha.* ...
La.*sintheta1+2.*z)+2.*(R+cosbeta.*costheta1.*La+La.*sinalpha.* ...
sinbeta.*sintheta1+(-1).*x).*(2.*cosalpha.*La.*sintheta2+2.*z)).*( ...
(sqrt(3).*R+(-1).*costheta3.*La.*sinbeta+sqrt(3).*La.*sinalpha.* ...
sinbeta.*sintheta3+(-1).*cosbeta.*La.*((-1).*sqrt(3).*costheta3+( ...
-1).*sinalpha.*sintheta3)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z) ...
+(-1).*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.* ...
sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta3+2.*z))).^(-1);

lamda3 = (-1).*((-1).*((-1).*(dpz+(1/2).*g.*(3.*Mb+2.*Mc)).*(2.*costheta1.* ...
La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.*sintheta1+2.*y)+dpy.*(2.* ...
cosalpha.*La.*sintheta1+2.*z)).*((R+(-1).*sqrt(3).*costheta2.*La.* ...
sinbeta+La.*sinalpha.*sinbeta.*sintheta2+cosbeta.*La.*(costheta2+ ...
sqrt(3).*sinalpha.*sintheta2)+2.*x).*(2.*cosalpha.*La.*sintheta1+ ...
2.*z)+2.*(R+cosbeta.*costheta1.*La+La.*sinalpha.*sinbeta.* ...
sintheta1+(-1).*x).*(2.*cosalpha.*La.*sintheta2+2.*z))+(2.*(dpz+( ...
1/2).*g.*(3.*Mb+2.*Mc)).*(R+cosbeta.*costheta1.*La+La.*sinalpha.* ...
sinbeta.*sintheta1+(-1).*x)+dpx.*(2.*cosalpha.*La.*sintheta1+2.*z) ...
).*(((-1).*sqrt(3).*R+(-1).*costheta2.*La.*sinbeta+(-1).*sqrt(3).* ...
La.*sinalpha.*sinbeta.*sintheta2+(-1).*cosbeta.*La.*(sqrt(3).* ...
costheta2+(-1).*sinalpha.*sintheta2)+2.*y).*(2.*cosalpha.*La.* ...
sintheta1+2.*z)+(-1).*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.* ...
La.*sinalpha.*sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta2+2.*z))) ...
.*((((-1).*sqrt(3).*R+(-1).*costheta2.*La.*sinbeta+(-1).*sqrt(3).* ...
La.*sinalpha.*sinbeta.*sintheta2+(-1).*cosbeta.*La.*(sqrt(3).* ...
costheta2+(-1).*sinalpha.*sintheta2)+2.*y).*(2.*cosalpha.*La.* ...
sintheta1+2.*z)+(-1).*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.* ...
La.*sinalpha.*sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta2+2.*z)) ...
.*((R+sqrt(3).*costheta3.*La.*sinbeta+La.*sinalpha.*sinbeta.* ...
sintheta3+cosbeta.*La.*(costheta3+(-1).*sqrt(3).*sinalpha.* ...
sintheta3)+2.*x).*(2.*cosalpha.*La.*sintheta1+2.*z)+2.*(R+ ...
cosbeta.*costheta1.*La+La.*sinalpha.*sinbeta.*sintheta1+(-1).*x).* ...
(2.*cosalpha.*La.*sintheta3+2.*z))+(-1).*((R+(-1).*sqrt(3).* ...
costheta2.*La.*sinbeta+La.*sinalpha.*sinbeta.*sintheta2+cosbeta.* ...
La.*(costheta2+sqrt(3).*sinalpha.*sintheta2)+2.*x).*(2.*cosalpha.* ...
La.*sintheta1+2.*z)+2.*(R+cosbeta.*costheta1.*La+La.*sinalpha.* ...
sinbeta.*sintheta1+(-1).*x).*(2.*cosalpha.*La.*sintheta2+2.*z)).*( ...
(sqrt(3).*R+(-1).*costheta3.*La.*sinbeta+sqrt(3).*La.*sinalpha.* ...
sinbeta.*sintheta3+(-1).*cosbeta.*La.*((-1).*sqrt(3).*costheta3+( ...
-1).*sinalpha.*sintheta3)+2.*y).*(2.*cosalpha.*La.*sintheta1+2.*z) ...
+(-1).*(2.*costheta1.*La.*sinbeta+(-2).*cosbeta.*La.*sinalpha.* ...
sintheta1+2.*y).*(2.*cosalpha.*La.*sintheta3+2.*z))).^(-1);

lamda = [lamda1 lamda2 lamda3];
