% function[Q]=torques(q, dq, dp, lamda, alpha, beta)

% Motor Torque function
%
% This function is used by the IDHtraj function to calculate motor torques for the whole trajectory in one operation. 
% It can be used on a point by point basis however. 
%
% Input parameters:
% q = matrix of travelling plate position and motor angles.
% dq = matrix of travelling plate velocity and motor angular velocity.
% dp = matrix of first derivative of canonical momentas.
%
% Outputs:
% Q = motor torques for entire time history.
%
% Author: Ben Hawkey 2000.

function[Q]=torques(q, dq, dp, lamda, alpha, beta)

InitArms
InitDynamics

x = q(:,1);
y = q(:,2);
z = q(:,3);
theta1 = q(:,4);
theta2 = q(:,5);
theta3 = q(:,6);
dx = dq(:,1);
dy = dq(:,2);
dz = dq(:,3);
dtheta1 = dq(:,4);
dtheta2 = dq(:,5);
dtheta3 = dq(:,6);
dptheta1 = dp(:,4);
dptheta2 = dp(:,5);
dptheta3 = dp(:,6);

lamda1 = lamda(:,1);
lamda2 = lamda(:,2);
lamda3 = lamda(:,3);

R = Ra-Rb;

cosalpha = cos(alpha);
sinalpha = sin(alpha);
cosbeta = cos(beta);
sinbeta = sin(beta);

costheta1 = cos(theta1);
sintheta1 = sin(theta1);
costheta2 = cos(theta2);
sintheta2 = sin(theta2);
costheta3 = cos(theta3);
sintheta3 = sin(theta3);

Q4 = dptheta1+(-1/2).*cosalpha.*costheta1.*g.*La.*(2.*H.*Ma+Mb+2.*Mj)+( ...
1/12).*((-2).*cosalpha.*dtheta1.*dz.*La.*Mb.*sintheta1+(-1).*dy.* ...
La.*Mb.*(2.*costheta1.*dtheta1.*sinbeta+(-2).*cosbeta.*dtheta1.* ...
sinalpha.*sintheta1)+(-2).*dtheta1.*dx.*La.*Mb.*((-1).*cosbeta.* ...
costheta1+(-1).*sinalpha.*sinbeta.*sintheta1))+2.*La.*lamda1.*( ...
costheta1.*R.*sinalpha.*sinbeta+(-1).*cosbeta.*R.*sintheta1+((-1) ...
.*costheta1.*sinalpha.*sinbeta+cosbeta.*sintheta1).*x+(-1).*( ...
cosbeta.*costheta1.*sinalpha+sinbeta.*sintheta1).*y+cosalpha.* ...
costheta1.*z);

Q5 = dptheta2+(-1/2).*cosalpha.*costheta2.*g.*La.*(2.*H.*Ma+Mb+2.*Mj)+( ...
1/12).*((-2).*cosalpha.*dtheta2.*dz.*La.*Mb.*sintheta2+(-1).* ...
dtheta2.*dx.*La.*Mb.*(costheta2.*(cosbeta+(-1).*sqrt(3).*sinbeta)+ ...
sinalpha.*(sqrt(3).*cosbeta+sinbeta).*sintheta2)+(-1).*dtheta2.* ...
dy.*La.*Mb.*((-1).*costheta2.*(sqrt(3).*cosbeta+sinbeta)+(-1).* ...
sinalpha.*((-1).*cosbeta+sqrt(3).*sinbeta).*sintheta2))+La.* ...
lamda2.*((cosbeta.*(sqrt(3).*costheta2.*sinalpha+(-1).*sintheta2)+ ...
sinbeta.*(costheta2.*sinalpha+sqrt(3).*sintheta2)).*x+(sinbeta.*(( ...
-1).*sqrt(3).*costheta2.*sinalpha+sintheta2)+cosbeta.*(costheta2.* ...
sinalpha+sqrt(3).*sintheta2)).*y+2.*(costheta2.*R.*sinalpha.* ...
sinbeta+(-1).*cosbeta.*R.*sintheta2+cosalpha.*costheta2.*z));

Q6 = dptheta3+(-1/2).*cosalpha.*costheta3.*g.*La.*(2.*H.*Ma+Mb+2.*Mj)+( ...
1/12).*((-2).*cosalpha.*dtheta3.*dz.*La.*Mb.*sintheta3+(-1).* ...
dtheta3.*dx.*La.*Mb.*(costheta3.*(cosbeta+sqrt(3).*sinbeta)+(-1).* ...
sinalpha.*(sqrt(3).*cosbeta+(-1).*sinbeta).*sintheta3)+(-1).*dy.* ...
La.*Mb.*(sqrt(3).*cosbeta.*costheta3.*dtheta3+(-1).*costheta3.* ...
dtheta3.*sinbeta+cosbeta.*dtheta3.*sinalpha.*sintheta3+sqrt(3).* ...
dtheta3.*sinalpha.*sinbeta.*sintheta3))+La.*lamda3.*((-1).*( ...
cosbeta.*(sqrt(3).*costheta3.*sinalpha+sintheta3)+sinbeta.*((-1).* ...
costheta3.*sinalpha+sqrt(3).*sintheta3)).*x+(sinbeta.*(sqrt(3).* ...
costheta3.*sinalpha+sintheta3)+cosbeta.*(costheta3.*sinalpha+(-1) ...
.*sqrt(3).*sintheta3)).*y+2.*(costheta3.*R.*sinalpha.*sinbeta+(-1) ...
.*cosbeta.*R.*sintheta3+cosalpha.*costheta3.*z));


Q = [Q4, Q5, Q6];
